#include "stdafx.h"
#include "..\..\DuiLib\UIlib.h"
#include "NeMessengerDlg.h"
#include "main_frame.hpp"
#include "ChatSearchList.hpp"
#include <algorithm>
#include "DuiDefine.h"

namespace DuiLib
{

const int kFriendListItemNormalHeight = 28;
const int kFriendListItemSelectedHeight = 28;

const TCHAR* const kOperatorPannelControlName = _T("operation");

CChatSearchList::CChatSearchList(CPaintManagerUI& paint_manager, HWND hWnd)
	: root_node_(NULL)
	, delay_deltaY_(0)
	, delay_number_(0)
	, delay_left_(0)
	, level_expand_image_(_T("<i list_icon_b.png>"))
	, level_collapse_image_(_T("<i list_icon_a.png>"))
	, level_text_start_pos_(0)
	, text_padding_(0, 0, 0, 0)
	, paint_manager_(paint_manager)
{
	SetItemShowHtml(true);

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
	m_hParent = hWnd;
}

CChatSearchList::~CChatSearchList()
{
	if (root_node_)
		delete root_node_;

	root_node_ = NULL;
}

HWND CChatSearchList::GetParentHwnd()
{
	return m_hParent;
}

bool CChatSearchList::Add(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::Add(pControl);
}

bool CChatSearchList::AddAt(CControlUI* pControl, int iIndex)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::AddAt(pControl, iIndex);
}

bool CChatSearchList::Remove(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::Remove(pControl);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

bool CChatSearchList::RemoveAt(int iIndex)
{
	CControlUI* pControl = GetItemAt(iIndex);
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::RemoveAt(iIndex);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

void CChatSearchList::RemoveAll()
{
	CListUI::RemoveAll();
	int nCount = root_node_->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = root_node_->child(i);
		RemoveNode(child);
	}
	delete root_node_;

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
}

void CChatSearchList::DoEvent(TEventUI& event) 
{
	if (!IsMouseEnabled() && event.Type > UIEVENT__MOUSEBEGIN && event.Type < UIEVENT__MOUSEEND)
	{
		if (m_pParent != NULL)
			m_pParent->DoEvent(event);
		else
			CVerticalLayoutUI::DoEvent(event);
		return;
	}

	if (event.Type == UIEVENT_TIMER && event.wParam == SCROLL_TIMERID)
	{
		if (delay_left_ > 0)
		{
			--delay_left_;
			SIZE sz = GetScrollPos();
			LONG lDeltaY =  0;
            if(delay_number_ > 0)
            {
                lDeltaY = (LONG)(CalculateDelay((double)delay_left_ / delay_number_) * delay_deltaY_);
            }
			if ((lDeltaY > 0 && sz.cy != 0)  || (lDeltaY < 0 && sz.cy != GetScrollRange().cy ))
			{
				sz.cy -= lDeltaY;
				SetScrollPos(sz);
				return;
			}
		}
		delay_deltaY_ = 0;
		delay_number_ = 0;
		delay_left_ = 0;
		m_pManager->KillTimer(this, SCROLL_TIMERID);
		return;
	}
	if (event.Type == UIEVENT_SCROLLWHEEL)
	{
		LONG lDeltaY = 0;
		if (delay_number_ > 0)
			lDeltaY =  (LONG)(CalculateDelay((double)delay_left_ / delay_number_) * delay_deltaY_);
		switch (LOWORD(event.wParam))
		{
		case SB_LINEUP:
			if (delay_deltaY_ >= 0)
				delay_deltaY_ = lDeltaY + 8;
			else
				delay_deltaY_ = lDeltaY + 12;
			break;
		case SB_LINEDOWN:
			if (delay_deltaY_ <= 0)
				delay_deltaY_ = lDeltaY - 8;
			else
				delay_deltaY_ = lDeltaY - 12;
			break;
		}
		if
			(delay_deltaY_ > 100) delay_deltaY_ = 100;
		else if
			(delay_deltaY_ < -100) delay_deltaY_ = -100;

		delay_number_ = (DWORD)sqrt((double)abs(delay_deltaY_)) * 5;
		delay_left_ = delay_number_;
		m_pManager->SetTimer(this, SCROLL_TIMERID, 50U);
		return;
	}
	if(event.Type == UIEVENT_RBUTTONUP)
	{
		m_pManager->SendNotify(this, DUI_MSGTYPE_ITEMCLICK, UIEVENT_RBUTTONUP, -1);
	}

	CListUI::DoEvent(event);
}

Node* CChatSearchList::GetRoot()
{
	return root_node_;
}

const TCHAR* const kLogoButtonControlName = _T("logo");
const TCHAR* const kLogoContainerControlName = _T("logo_container");
const TCHAR* const kNickNameControlName = _T("nickname");
const TCHAR* const kDescriptionControlName = _T("description");

static bool OnLogoButtonEvent(void* event) {
	TEventUI *pEvent = (TEventUI*)event;
	static CString strOldImage = _T("");
    if( ((TEventUI*)event)->Type == UIEVENT_BUTTONDOWN ) {
        CControlUI* pButton = ((TEventUI*)event)->pSender;
        if( pButton != NULL ) {
             CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			 if( pListElement != NULL ) {
				 CChatSearchList *pThis = (CChatSearchList *)pListElement->GetOwner();
				 CButtonUI* logo_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo")));
				 CRect rcItem = pButton->GetPos();
				 Node* pNode = (Node*)pListElement->GetTag();
				 if (pNode != NULL)
				 {
                     CRect rcRootWnd(0, 0, 0, 0);
                     GetWindowRect(pThis->paint_manager_.GetPaintWindow(),&rcRootWnd);
					 RECT rcAround={0};
					 rcAround.right = rcItem.right + rcRootWnd.left;
					 rcAround.left = rcItem.left + rcRootWnd.left ;
					 rcAround.top = rcItem.top +  rcRootWnd.top;
					 rcAround.bottom = rcItem.bottom + rcRootWnd.top ;
					 if (GET_CONFIG()->EnableFunction(_T("UI_03")))
					 {
						GET_MESSENGER_DLG()->GetProfileDlgController()->CreateProfileDlg(pNode->data().value.GetData(), rcAround, pThis->GetParentHwnd());
					 }
				 }

				 CString strImage = GetAppDUIImgDir() + _T("\\btn_profile_icon.png");				
				 logo_button->SetBkImage(strImage);
				 pListElement->DoEvent(*(TEventUI*)event);
			 }			
        }
    }
	else  if( ((TEventUI*)event)->Type == UIEVENT_MOUSEENTER )	
	{
		CControlUI* pButton = ((TEventUI*)event)->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			if( pListElement != NULL ) 
			{
				CChatSearchList *pThis = (CChatSearchList *)pListElement->GetOwner();
				CButtonUI* logo_hot_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo")));
				if (logo_hot_button!=NULL )
				{			
					Node* pNode = (Node*)pListElement->GetTag();
					CString strImage = GetAppDUIImgDir() +_T("\\img_main_mouse_move.png");
					strOldImage = logo_hot_button->GetBkImage();
					logo_hot_button->SetBkImage(strImage);
				}
				pListElement->DoEvent(*pEvent);
			}
		}
	}
	else if(((TEventUI*)event)->Type== UIEVENT_MOUSELEAVE )
	{
		CControlUI* pButton =((TEventUI*)event)->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			if( pListElement != NULL ) 
			{
				CChatSearchList *pThis = (CChatSearchList *)pListElement->GetOwner();
				CButtonUI* log_hot_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo")));
				if (log_hot_button!=NULL)
				{
					log_hot_button->SetBkImage(strOldImage);
				}
				pListElement->DoEvent(*pEvent);
			}
		}
	}
    return true;
}

Node* CChatSearchList::AddNode(const ChatSearchItemInfo& item, int nCompareData, Node* parent, int nTopPadding)
{
	if (!parent)
		parent = root_node_;

	TCHAR szBuf[MAX_PATH] = {0};

    CListContainerElementUI* pListElement = NULL;
    if( !m_dlgBuilder.GetMarkup()->IsValid() ) {
        pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("chat_search_item.xml"), (UINT)0, NULL, &paint_manager_));
    }
    else {
        pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, &paint_manager_));
    }
    if (pListElement == NULL)
        return NULL;

	//pListElement->SetCompareData(nCompareData);

	Node* node = new Node;

	node->SetComapreData(nCompareData);

	node->data().level_ = parent->data().level_ + 1;
	if (item.folder)
		node->data().has_child_ = !item.empty;
	else
		node->data().has_child_ = false;

	node->data().folder_ = item.folder;

	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;

	node->data().text_ = item.nick_name;
	node->data().value = item.id;
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	if (parent != root_node_ && !parent->data().list_elment_->IsVisible())
		pListElement->SetVisible(false);

	CDuiRect rcPadding = text_padding_;
	rcPadding.top += nTopPadding;
	for (int i = 0; i < node->data().level_; ++i)
	{
		rcPadding.left += level_text_start_pos_;		
	}
	pListElement->SetPadding(rcPadding);

	CButtonUI* log_button = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
	if (log_button != NULL)
	{
		if (!item.folder && !item.logo.IsEmpty())
		{
#if defined(UNDER_WINCE)
			_stprintf(szBuf, _T("%s"), item.logo);
#else
			_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.logo);
#endif
			
			log_button->SetBkImage(szBuf);
		}
		else
		{
			CContainerUI* logo_container = static_cast<CContainerUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoContainerControlName));
			if (logo_container != NULL)
				logo_container->SetVisible(false);
		}
        log_button->SetTag((UINT_PTR)pListElement);
        log_button->OnEvent += MakeDelegate(&OnLogoButtonEvent);
	}

	CDuiString html_text;
	if (node->data().has_child_)
	{
		if (node->data().child_visible_)
			html_text += level_expand_image_;
		else
			html_text += level_collapse_image_;

#if defined(UNDER_WINCE)
		_stprintf(szBuf, _T("<x %d>"), 10);
#else
		_stprintf_s(szBuf, MAX_PATH - 1, _T("<x %d>"), 10);
#endif
		html_text += szBuf;
	}

	if (item.folder)
	{
		html_text += node->data().text_;
	}
	else
	{
#if defined(UNDER_WINCE)
		_stprintf(szBuf, _T("%s"), item.nick_name);
#else
		_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.nick_name);
#endif
		html_text += szBuf;
	}

	CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kNickNameControlName));
	if (nick_name != NULL)
	{
		if (item.folder)
			nick_name->SetFixedWidth(0);

		nick_name->SetShowHtml(true);
		nick_name->SetText(html_text);
	}

	if (!item.folder && !item.description.IsEmpty())
	{
		CLabelUI* description = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kDescriptionControlName));
		if (description != NULL)
		{
#if defined(UNDER_WINCE)
			_stprintf(szBuf, _T("<x 20><c #808080>%s</c>"), item.description);
#else
			_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.description);
#endif
			description->SetShowHtml(true);
			description->SetText(szBuf);
		}
	}

	pListElement->SetFixedHeight(kFriendListItemNormalHeight);
	pListElement->SetTag((UINT_PTR)node);
	CButtonUI* btn_add = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(pListElement, kChatAddBtn));
	if (btn_add != NULL)
	{
		btn_add->SetTag((UINT_PTR)node);
	}

	int index = 0;
	int innerIndex = 0;
	int iSumChildCount = 0;
	if (parent->has_children())
	{
		Node* prev = parent->get_last_child();
		index = prev->data().list_elment_->GetIndex() + 1;
		for (int i = 0; i < parent->num_children(); i++)
		{
			Node* pNode = parent->child(i);
			if(pNode->GetCompareData() > nCompareData)
			{
				index = pNode->data().list_elment_->GetIndex();
				break;
			}
			else
			{
				iSumChildCount += pNode->num_children();
			}
		}

		if (parent == root_node_)
		{
			innerIndex = index - iSumChildCount;
		}
		else
		{
			innerIndex = index - parent->data().list_elment_->GetIndex() - 1;
		}
		
	}
	else 
	{
		if (parent == root_node_)
			index = 0;
		else
		{
			index = parent->data().list_elment_->GetIndex() + 1;
		}
		innerIndex = 0;
	}
	if (!CListUI::AddAt(pListElement, index))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}

	parent->insert_child(node, innerIndex);
	return node;
}

bool CChatSearchList::RemoveNode(Node* node)
{
	if (!node || node == root_node_) return false;

	int nCount = node->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = node->child(i);
		RemoveNode(child);
	}

	CListUI::Remove(node->data().list_elment_);
	node->parent()->remove_child(node);
	delete node;

	return true;
}

void CChatSearchList::SetChildVisible(Node* node, bool visible)
{
	if (!node || node == root_node_)
		return;

	if (node->data().child_visible_ == visible)
		return;

	node->data().child_visible_ = visible;

	TCHAR szBuf[MAX_PATH] = {0};
	CDuiString html_text;
	if (node->data().has_child_)
	{
		if (node->data().child_visible_)
			html_text += level_expand_image_;
		else
			html_text += level_collapse_image_;

#if defined(UNDER_WINCE)
		_stprintf(szBuf, _T("<x %d>"), 10);
#else
		_stprintf_s(szBuf, MAX_PATH - 1, _T("<x %d>"), 10);
#endif
		html_text += szBuf;

		html_text += node->data().text_;

		CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));
		if (nick_name != NULL)
		{
			nick_name->SetShowHtml(true);
			nick_name->SetText(html_text);
		}
	}

	if (!node->data().list_elment_->IsVisible())
		return;

	if (!node->has_children())
		return;

	Node* begin = node->child(0);
	Node* end = node->get_last_child();
	for (int i = begin->data().list_elment_->GetIndex(); i <= end->data().list_elment_->GetIndex(); ++i)
	{
		CControlUI* control = GetItemAt(i);
		if (_tcsicmp(control->GetClass(), _T("ListContainerElementUI")) == 0)
		{
			if (visible) 
			{
				Node* local_parent = ((Node*)control->GetTag())->parent();
				if (local_parent->data().child_visible_ && local_parent->data().list_elment_->IsVisible())
				{
					control->SetVisible(true);
				}
			}
			else
			{
				control->SetVisible(false);
			}
		}
	}
}

bool CChatSearchList::CanExpand(Node* node) const
{
	if (!node || node == root_node_)
		return false;

	return node->data().has_child_;
}

list<Node*> CChatSearchList::GetSelectedItems()
{
	list<Node*> listRet;
	list<int> listSelected = CListUI::GetSelectedItems();
	for(list<int>::iterator it = listSelected.begin();
		it != listSelected.end(); it++)
	{
		CControlUI* pControl = GetItemAt(*it);
		//Node* node = NULL;
		if( pControl != NULL) 
		{
			Node* node = (Node*)pControl->GetTag();
			if(node != NULL)
			{
				listRet.push_back(node);
			}
		}
	}
	
	return listRet;
}

void CChatSearchList::SetSelectViewModal( CControlUI* pControl, bool bSelect)
{
	int nHeight = bSelect ? kFriendListItemSelectedHeight : kFriendListItemNormalHeight;
	CListContainerElementUI* pFriendListItem = static_cast<CListContainerElementUI*>(pControl);
	Node* node = (Node*)pControl->GetTag();
	if ((pFriendListItem != NULL) && (node != NULL) && !node->folder())
	{
		pFriendListItem->SetFixedHeight(nHeight);
		CContainerUI* pOperatorPannel = static_cast<CContainerUI*>(m_pManager->FindSubControlByName(pFriendListItem, kOperatorPannelControlName));
		if (pOperatorPannel != NULL)
		{
			pOperatorPannel->SetVisible(bSelect);
		}
	}
}



} // namespace DuiLib